* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

********************************************************************************;
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 5a               *******;
*******             CHANGE "filesave" to your folder path                *******;
********************************************************************************;

* Location for original files for these models  change this path;
* \\Client\ precedes path in Virtual Desktop outside H drive;
%LET filesave=C:\Dropbox\23_PSQF6270\PSQF6270_Example5; 
LIBNAME filesave "&filesave.";

* Import Example 5a multivariate data into work library and stack it;
DATA work.Example5; SET filesave.PSQF6270_Example5Wide; * Adding dummy codes while stacking;
     DV="1.Kid"; kid=1; mom=0; dad=0; marital=KidMarital; OUTPUT;
     DV="2.Mom"; kid=0; mom=1; dad=0; marital=MomMarital; OUTPUT;
     DV="3.Dad"; kid=0; mom=0; dad=1; marital=DadMarital; OUTPUT;
     LABEL DV=      "DV: 1K,2M,3D"
           kid=     "kid: Is Adult Child (0=no, 1=yes)"
           mom=     "mom: Is Mother (0=no, 1=yes)"
           dad=     "dad: Is Father (0=no, 1=yes)"
           marital= "marital: Marital Gender Attitudes Mean (1-4)"; 
     DROP KidMarital MomMarital DadMarital; * Remove original outcomes;
RUN; * Remove missing predictors or row-specific (will happen anyway);
DATA work.Example5; SET work.Example5;
     IF NMISS(KidBoy,KidEd12,MomEd12,DadEd12,marital)>0 THEN DELETE;
RUN;


TITLE "SAS Descriptives for Marital Outcome by Person";
PROC MEANS DATA=work.Example5; CLASS DV; VAR marital; RUN; 
PROC SORT DATA=work.Example5; BY DV;
PROC UNIVARIATE NOPRINT DATA=work.Example5; 
     BY DV; VAR marital; 
     HISTOGRAM marital / MIDPOINTS=1 TO 4 BY 0.5;
RUN; TITLE;

********************************************************************************;
*******                   BEGIN EXAMPLE 5 PART 1 MODELS                  *******;
********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\PSQF6270_Example5_SAS_Output_Part1mixed.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

*** 2 Versions of Baseline Model 0 ***;
TITLE1 "SAS Empty Means, Unstructured Variance Models for Marital Attitudes";

TITLE2 "SAS Model 0a: General Intercept (Dad=Ref DV) using 2 Dummy Codes";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
* CLASS is for ID variables and program-categorical predictors;
  CLASS FamilyID DV;  * Fixed intercept will be for dad (as omitted);
  MODEL marital = kid mom / SOLUTION DDFM=Satterthwaite;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID;
  CONTRAST "DF=2 Intercept Diff"          kid 1, mom 1;
  ESTIMATE "Kid Intercept (Dad+diff)"     intercept 1 kid 1 mom 0;
  ESTIMATE "Mom Intercept (Dad+diff)"     intercept 1 kid 0 mom 1;
  ESTIMATE "Kid vs. Mom: Intercept Diff"  kid -1 mom 1; 
RUN; TITLE2;

TITLE2 "SAS Model 0b: DV-Specific Intercepts using All 3 Dummy Codes";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
* CLASS is for ID variables and program-categorical predictors;
  CLASS FamilyID DV; * NOINT removes general fixed intercept;
  MODEL marital = kid mom dad / NOINT SOLUTION DDFM=Satterthwaite;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID; * UN = unstructured R;
  CONTRAST "DF=2 Intercept Diff"          kid -1 mom 1, kid -1 dad 1;
  ESTIMATE "Kid vs. Mom: Intercept Diff"  kid -1 mom 1; 
  ESTIMATE "Kid vs. Dad: Intercept Diff"  kid -1 dad 1;
  ESTIMATE "Mom vs. Dad: Intercept Diff"  mom -1 dad 1;
RUN; TITLE2;


TITLE "SAS Model 1: DV-Specific Intercepts -- Add Kid Gender";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
  CLASS FamilyID DV;  * CLASS is for ID variables;
  MODEL marital = kid mom dad kid*KidBoy mom*KidBoy dad*KidBoy 
                   / NOINT SOLUTION DDFM=Satterthwaite;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID; 
  CONTRAST "DF=2 Diff in KidBoy Slope?" kid*KidBoy -1 Mom*KidBoy 1, kid*KidBoy -1 Dad*KidBoy 1; 
  ESTIMATE "Kid vs. Mom: KidBoy Slope Diff"  kid*KidBoy -1 Mom*KidBoy 1; 
  ESTIMATE "Kid vs. Dad: KidBoy Slope Diff"  kid*KidBoy -1 Dad*KidBoy 1;
  ESTIMATE "Mom vs. Dad: KidBoy Slope Diff"  Mom*KidBoy -1 Dad*KidBoy 1;
  ESTIMATE "Parent KidBoy Slope" Mom*KidBoy 1 Dad*KidBoy 1 / DIVISOR=2; 
  ESTIMATE "Kid vs. Parents: KidBoy Slope Diff"  Kid*KidBoy -2  Mom*KidBoy 1 Dad*KidBoy 1 / DIVISOR=2;
RUN; TITLE;


TITLE "SAS Model 2: DV-Specific Intercepts -- KidBoy on Kid Only, Add Own Educ";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
  CLASS FamilyID DV;  * CLASS is for ID variables;
  MODEL marital = kid mom dad kid*KidBoy kid*KidEd12 mom*MomEd12 dad*DadEd12 
                    / NOINT SOLUTION DDFM=Satterthwaite;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID; 
RUN; TITLE;


TITLE "SAS Model 3: DV-Specific Intercepts -- Add Dad Educ (Control for Own Educ)";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
  CLASS FamilyID DV;  * CLASS is for ID variables;
  MODEL marital = kid mom dad kid*KidBoy Kid*KidEd12 Mom*MomEd12 Dad*DadEd12 
                  Kid*DadEd12 Mom*DadEd12 / NOINT SOLUTION DDFM=Satterthwaite RESIDUAL;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID; * RESIDUAL requests plots of residuals;
  ESTIMATE "Kid vs. Mom: DadEd12 Slope Diff"  kid*DadEd12 -1 mom*DadEd12 1; 
  ESTIMATE "Kid vs. Dad: DadEd12 Slope Diff"  kid*DadEd12 -1 dad*DadEd12 1;
  ESTIMATE "Mom vs. Dad: DadEd12 Slope Diff"  mom*DadEd12 -1 dad*DadEd12 1;
RUN; TITLE;


* Close external file of saved results;
ODS RTF CLOSE;

********************************************************************************;
*******                   BEGIN EXAMPLE 5 PART 2 MODELS                  *******;
********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\PSQF6270_Example5_SAS_Output_Part2mixed.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

* Export original wide format to Mplus;
DATA work.ForMplus; SET filesave.PSQF6270_Example5Wide;
* Fixing any missing values;
  ARRAY vars(8) FamilyID KidBoy KidEd12 MomEd12 DadEd12 KidMarital MomMarital DadMarital;
     DO i=1 TO 8; IF vars(i)=. THEN vars(i)=-999; END; DROP i;
RUN;
PROC EXPORT DATA=work.ForMplus OUTFILE= "&filesave.\PSQF6270_Example5Wide.csv" 
     DBMS=CSV REPLACE; PUTNAMES=NO; RUN;

TITLE1 "SAS Model 4: DV-Specific Intercepts -- Add Mom Educ (Controlling for Own+Dad Educ)";
TITLE2 "To match path model, switch to ML estimation, robust SEs via EMPIRICAL"; 
TITLE3 "Satterthwaite DF not allowed with EMPIRICAL, so switch to residual (N-k)";
PROC MIXED DATA=work.Example5 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=ML EMPIRICAL;
  CLASS FamilyID DV;  * CLASS is for ID variables;
  MODEL marital = kid mom dad kid*KidBoy  kid*KidEd12 mom*MomEd12 dad*DadEd12 
                  kid*DadEd12 mom*DadEd12 kid*MomEd12 dad*MomEd12 
                   / NOINT SOLUTION DDFM=RESIDUAL /* OUTPM saves y-hat from fixed effects */
                     OUTPM=work.PredFinal RESIDUAL; * RESIDUAL requests plots of residuals;
  REPEATED DV / R RCORR TYPE=UN SUBJECT=FamilyID; 
  ESTIMATE "Kid vs. Mom: DadEd12 Slope Diff"  kid*DadEd12 -1 mom*DadEd12 1; 
  ESTIMATE "Kid vs. Dad: DadEd12 Slope Diff"  kid*DadEd12 -1 dad*DadEd12 1;
  ESTIMATE "Mom vs. Dad: DadEd12 Slope Diff"  mom*DadEd12 -1 dad*DadEd12 1;
  ESTIMATE "Kid vs. Mom: MomEd12 Slope Diff"  kid*MomEd12 -1 mom*MomEd12 1; 
  ESTIMATE "Kid vs. Dad: MomEd12 Slope Diff"  kid*MomEd12 -1 dad*MomEd12 1;
  ESTIMATE "Mom vs. Dad: MomEd12 Slope Diff"  mom*MomEd12 -1 dad*MomEd12 1;
RUN; TITLE1; TITLE2; TITLE3;
* Save corr of pred and actual marital attitudes to square as R2;
TITLE "Correlation of Predicted and Actual Marital Attitudes by DV";
PROC SORT DATA=work.PredFinal; BY DV FamilyID; RUN;
PROC CORR NOSIMPLE DATA=work.PredFinal OUT=work.Rpred; 
     BY DV; VAR marital; WITH pred; RUN; 
* Compute R2 in saved output;
DATA work.Rpred; SET work.Rpred;
     WHERE _TYPE_="CORR"; R2=marital*marital; RUN;
TITLE "R2 of Predicted and Actual Marital Attitudes by DV";
PROC PRINT NOOBS DATA=work.Rpred; ID DV; VAR R2; RUN; TITLE;

* Close external file of saved results;
ODS RTF CLOSE;
