// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 5                *******
*******             CHANGE "filesave" to your folder path                *******
********************************************************************************

// Defining global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive;
global filesave "C:\Dropbox\23_PSQF6270\PSQF6270_Example5"

// Import Example 5 wide Stata data
use "$filesave\PSQF6270_Example5Wide.dta", clear
   
// Rename variables with numeric suffix to use with reshape (old)(new)
rename (kidmarital mommarital dadmarital) (marital1 marital2 marital3)

// Stack data: list multivariate variables first, i(higher index) j(repeated)
reshape long marital, i(familyid) j(DVnum)

// Create per-outcome dummy codes 
gen kid=0
gen mom=0
gen dad=0
recode kid (0=1) if DVnum==1
recode mom (0=1) if DVnum==2
recode dad (0=1) if DVnum==3

// Label new variables
label variable DVnum   "DVnum: 1K,2M,3D"
label variable kid     "kid: Is Adult Child (0=no, 1=yes)"
label variable mom     "mom: Is Mother (0=no, 1=yes)"
label variable dad     "dad: Is Father (0=no, 1=yes)"
label variable marital "marital: Marital Gender Attitudes Mean (1-4)"

// Remove missing predictors or row-specific outcome (will happen anyway)
egen nummiss = rowmiss(kidboy kided12 momed12 daded12 marital)
drop if nummiss>0


********************************************************************************
*******                   BEGIN EXAMPLE 5 PART 1 MODELS                  *******
********************************************************************************

// Open external file to save results to
log using $filesave\PSQF6270_Example5_STATA_Output_Part1mixed.log, replace name(Example5_Part1mixed)

////// 2 Versions of Baseline Model 0 ///////
display "STATA Empty Means, Unstructured nolog Models for Marital Attitudes"

display "STATA Model 0a: General Intercept (Dad=Ref DV) using 2 Dummy Codes"
mixed marital c.kid c.mom,        /// Fixed intercept will be for dad (as omitted)
      || familyid: , noconstant   /// This NOCONSTANT removes family random intercept
      nolog reml residuals(unstructured,t(DVnum))       /// Unstructured R matrix by DV
      difficult dfmethod(satterthwaite) dftable(pvalue)  // Use Satterthwaite denominator DF
display "-2LL=" e(ll)*-2          // Print -2LL for model
estat wcorrelation, covariance    // R matrix
estat wcorrelation                // RCORR matrix  
lincom _cons*1  + c.kid*1, small  // Kid Intercept (Dad + diff)
lincom _cons*1  + c.mom*1, small  // Mom Intercept (Dad + diff)
lincom c.kid*-1 + c.mom*1, small  // Kid vs. Mom: Intercept Diff

display "STATA Model 0b: DV-Specific Intercepts using All 3 Dummy Codes"
mixed marital c.kid c.mom c.dad, noconstant  /// This NOCONSTANT removes general fixed intercept
       || familyid: , noconstant             /// This NOCONSTANT removes family random intercept
      nolog reml residuals(unstructured,t(DVnum))       /// Unstructured R matrix by DV
      difficult dfmethod(satterthwaite) dftable(pvalue)  // Use Satterthwaite denominator DF
display "-2LL= " e(ll)*-2               // Print -2LL for model
estat wcorrelation, covariance          // R matrix
estat wcorrelation                      // RCORR matrix  
test (c.kid=c.mom)(c.kid=c.dad), small  // DF=2 Intercept Diff (small = use denominator DF)
lincom c.kid*-1 + c.mom*1, small        // Kid vs. Mom: Intercept Diff
lincom c.kid*-1 + c.dad*1, small        // Kid vs. Dad: Intercept Diff
lincom c.mom*-1 + c.dad*1, small        // Mom vs. Dad: Intercept Diff
   
          
display "STATA Model 1: DV-Specific Intercepts -- Add Kid Gender"
mixed marital c.kid c.mom c.dad c.kid#c.kidboy c.mom#c.kidboy c.dad#c.kidboy, noconstant ///
      || familyid: , noconstant     /// This NOCONSTANT removes family random intercept
      nolog reml residuals(unstructured,t(DVnum))       /// Unstructured R matrix by DV
      difficult dfmethod(satterthwaite) dftable(pvalue)  // Use Satterthwaite denominator DF
display "-2LL= " e(ll)*-2           // Print -2LL for model
estat wcorrelation, covariance      // R matrix
estat wcorrelation                  // RCORR matrix  
// DF=2 Diff in Kidboy Slope
test (c.kid#c.kidboy=c.mom#c.kidboy)(c.kid#c.kidboy=c.dad#c.kidboy), small  
lincom c.kid#c.kidboy*-1 + c.mom#c.kidboy*1, small // Kid vs. Mom: Kidboy Slope Diff
lincom c.kid#c.kidboy*-1 + c.dad#c.kidboy*1, small // Kid vs. Dad: Kidboy Slope Diff
lincom c.mom#c.kidboy*-1 + c.dad#c.kidboy*1, small // Mom vs. Dad: Kidboy Slope Diff   
lincom 0.5*(c.mom#c.kidboy*1 + c.dad#c.kidboy*1), small // Parent: Kidboy Slope
// Mom vs. Dad: Kidboy Slope Diff      
lincom 0.5*(c.kid#c.kidboy*-2 + c.mom#c.kidboy*1 + c.dad#c.kidboy*1), small   
          

display "STATA Model 2: DV-Specific Intercepts -- KidBoy on Kid Only, Add Own Education"
mixed marital c.kid c.mom c.dad c.kid#c.kidboy                      ///
      c.kid#c.kided12 c.mom#c.momed12 c.dad#c.daded12, noconstant   /// 
      || familyid: , noconstant   /// This NOCONSTANT removes family random intercept
      nolog reml residuals(unstructured,t(DVnum))       /// Unstructured R matrix by DV
      difficult dfmethod(satterthwaite) dftable(pvalue)  // Use Satterthwaite denominator DF
display "-2LL= " e(ll)*-2           // Print -2LL for model
estat wcorrelation, covariance      // R matrix
estat wcorrelation                  // RCORR matrix      


display "STATA Model 3: DV-Specific Intercepts -- Add Dad Educ (Control for Own Educ)"
mixed marital c.kid c.mom c.dad c.kid#c.kidboy c.kid#c.kided12 c.mom#c.momed12  /// 
      c.dad#c.daded12 c.kid#c.daded12 c.mom#c.daded12, noconstant               ///
      || familyid: , noconstant   /// This NOCONSTANT removes family random intercept
      nolog reml residuals(unstructured,t(DVnum))       /// Unstructured R matrix by DV
      difficult dfmethod(satterthwaite) dftable(pvalue)  // Use Satterthwaite denominator DF
display "-2LL= " e(ll)*-2           // Print -2LL for model
estat wcorrelation, covariance      // R matrix
estat wcorrelation                  // RCORR matrix     
lincom c.kid#c.daded12*-1 + c.mom#c.daded12*1, small // Kid vs. Mom: DadEd12 Slope Diff
lincom c.kid#c.daded12*-1 + c.dad#c.daded12*1, small // Kid vs. Dad: DadEd12 Slope Diff
lincom c.mom#c.daded12*-1 + c.dad#c.daded12*1, small // Mom vs. Dad: DadEd12 Slope Diff 
predict Model3pred, xb         // Save yhat from fixed effects
predict Model3res, rstandard   // Save "standardized" residuals from fixed effects
hist Model3res                 // Histogram of residuals (for normality)
//graph export "$filesave\STATA Model 3 Residual Histogram.png", replace
twoway (scatter Model3res Model3pred)  // Scatterplot by predicted (for constant nolog)  
//graph export "$filesave\STATA Model 3 Residual Scatterplot.png", replace        

                    
// Close external file of results
log close Example5_Part1mixed


********************************************************************************
*******                   BEGIN EXAMPLE 5 PART 2 MODELS                  *******
********************************************************************************

// Open external file to save results to
log using $filesave\PSQF6270_Example5_STATA_Output_Part2mixed.log, replace name(Example5_Part2mixed)

display "STATA Model 4: DV-Specific Intercepts -- Add Mom Educ (Controlling for Own+Dad Educ)" 
display "To match path model in Part 2, switch to ML estimation, robust SEs" 
display "Satterthwaite DF not allowed with EMPIRICAL, so switch to residual (N-k)" 
mixed marital c.kid c.mom c.dad c.kid#c.kidboy c.kid#c.kided12 c.mom#c.momed12  ///
      c.dad#c.daded12 c.kid#c.daded12 c.mom#c.daded12                           /// 
      c.kid#c.momed12 c.dad#c.momed12, noconstant                               ///
      || familyid: , noconstant   /// This NOCONSTANT removes family random intercept
      nolog mle residuals(unstructured,t(DVnum))  /// Unstructured R matrix by DV
      difficult vce(robust)         // Use robust SEs, so no denominator DF allowed
display "-2LL= " e(ll)*-2           // Print -2LL for model
estat wcorrelation, covariance      // R matrix
estat wcorrelation                  // RCORR matrix  
predict Model4pred, xb  // Add column pred of predicted outcomes to data
lincom c.kid#c.daded12*-1 + c.mom#c.daded12*1 // Kid vs. Mom: DadEd12 Slope Diff
lincom c.kid#c.daded12*-1 + c.dad#c.daded12*1 // Kid vs. Dad: DadEd12 Slope Diff
lincom c.mom#c.daded12*-1 + c.dad#c.daded12*1 // Mom vs. Dad: DadEd12 Slope Diff     
lincom c.kid#c.momed12*-1 + c.mom#c.momed12*1 // Kid vs. Mom: MomEd12 Slope Diff
lincom c.kid#c.momed12*-1 + c.dad#c.momed12*1 // Kid vs. Dad: MomEd12 Slope Diff
lincom c.mom#c.momed12*-1 + c.dad#c.momed12*1 // Mom vs. Dad: MomEd12 Slope Diff     
// Get correlation of actual and predicted outcomes to form R2
pwcorr marital Model4pred if DV==1, sig 
display "DV=1 Kid R2= " r(rho)^2  // Print R2 relative to empty model
pwcorr marital Model4pred if DV==2, sig 
display "DV=2 Mom R2= " r(rho)^2  // Print R2 relative to empty model
pwcorr marital Model4pred if DV==3, sig 
display "DV=3 Dad R2= " r(rho)^2  // Print R2 relative to empty model

// Close external file of results
log close Example5_Part2mixed


/////////////////////////////////////////////////////////////////////////////////////////


// Defining global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive;
global filesave "C:\Dropbox\23_PSQF6270\PSQF6270_Example5"

// Import Example 5 wide STATA data
use "$filesave\PSQF6270_Example5Wide.dta", clear

// Example of how to export a .csv file for use in Mplus
// Replace all missing values with -999 for Mplus
mvencode _all, mv(-999)
// export delimited below: using lists the path and name of the new .csv file 
// replace means it will be replaced if a file already exists with that name 
// delimiter indicates a comma-delimited file 
// nolabel will save actual data (numbers) instead of any value labels included 
// novarnames tells it not to write the names to the top of the .csv file
export delimited using "$filesave\PSQF6270_Example5Wide_STATA.csv", ///
                 delimiter(",") replace nolabel novarnames
   
// Re-import Example 5 wide data (without missing value codes) for STATA analyses
use "$filesave\PSQF6270_Example5Wide.dta", clear   
   
// Open external file to save results to
log using $filesave\PSQF6270_Example5_STATA_Output_Part2path.log, replace name(Example5_Part2path)

* /// means continue the command + comment
*  // means comment only

display "STATA Model 3: Own Education + Dad Education a Predictor of Each Attitude"
display "Using SEM to create path analysis model estimated with ML on wide-format data"
sem                                          ///
   (kidmarit mommarit dadmarit <- _cons)     /// All intercepts estimated (by default)
   (kidmarit <- kidboy kided12)              /// Regressions: y outcomes ON x predictors 
   (mommarit <- momed12)                     ///
   (kidmarit mommarit dadmarit <- daded12),  ///
    var(e.kidmarit e.mommarit e.dadmarit)    /// All residual nologs estimated (by default)      
    covariance(e.kidmarit*e.mommarit         /// All pairwise residual covariances (not default)
               e.mommarit*e.dadmarit         ///
               e.kidmarit*e.dadmarit)        ///
    method(mlmv)                              // Full-information ML 
    lincom _b[mommarital:daded12] - _b[kidmarital:daded12] // Kid v. Mom: Dad Educ Effect Diff
    lincom _b[dadmarital:daded12] - _b[kidmarital:daded12] // Kid v. Dad: Dad Educ Effect Diff
    lincom _b[dadmarital:daded12] - _b[mommarital:daded12] // Mom v. Dad: Dad Educ Effect Diff
    sem, coeflegend                       // Print parameter labels, too (to use in lincom) 
    sem, standardized                     // Print fully standardized solution, too
    estat gof, stats(all)                 // Print fit statistics  
    display "LL for H1 Saturated Model= " e(critvalue_s)
    display "# of Estimated parameters= " e(df_m)
      display "-2LL= " e(ll)*-2             // Print -2LL for model
    estat eqgof                           // Print R2 per variable
    estat residuals                       // Print how far off each predicted covariance is
    estat mindices, minchi2(3.84) showpclass(all) // Print cheat codes to improve model fit at p<.05

     
display "STATA Model 4: Own + Dad + Mom Education a Predictor of Each Attitude"
display "Using SEM to create path analysis model estimated with ML on wide-format data"
sem                                          ///
   (kidmarit mommarit dadmarit <- _cons)     /// All intercepts estimated (by default)
   (kidmarit <- kidboy kided12)              /// Regressions: y outcomes ON x predictors 
   (kidmarit mommarit dadmarit <- daded12)   ///
   (kidmarit mommarit dadmarit <- momed12),  /// New effects go here
    var(e.kidmarit e.mommarit e.dadmarit)    /// All residual nologs estimated (by default)      
    covariance(e.kidmarit*e.mommarit         /// All pairwise residual covariances (not default)
               e.mommarit*e.dadmarit         ///
               e.kidmarit*e.dadmarit)        ///
    method(mlmv) vce(robust)                  // Full-information ML and robust SEs
    lincom _b[mommarital:daded12] - _b[kidmarital:daded12] // Kid v. Mom: Dad Educ Effect Diff
    lincom _b[dadmarital:daded12] - _b[kidmarital:daded12] // Kid v. Dad: Dad Educ Effect Diff
    lincom _b[dadmarital:daded12] - _b[mommarital:daded12] // Mom v. Dad: Dad Educ Effect Diff
    lincom _b[mommarital:momed12] - _b[kidmarital:momed12] // Kid v. Mom: Mom Educ Effect Diff
    lincom _b[dadmarital:momed12] - _b[kidmarital:momed12] // Kid v. Dad: Mom Educ Effect Diff
    lincom _b[dadmarital:momed12] - _b[mommarital:momed12] // Mom v. Dad: Mom Educ Effect Diff
    sem, coeflegend                       // Print parameter labels, too (to use in lincom) 
    sem, standardized                     // Print fully standardized solution, too
    estat gof, stats(all)                 // Print fit statistics  
    display "LL for H1 Saturated Model= " e(critvalue_s)
    display "# of Estimated parameters= " e(df_m)
      display "-2LL= " e(ll)*-2             // Print -2LL for model
    estat eqgof                           // Print R2 per variable
    estat residuals                       // Print how far off each predicted covariance is
    estat mindices, minchi2(3.84) showpclass(all) // Print cheat codes to improve model fit at p<.05

     
// Close external file of results
log close Example5_Part2path

